#!/usr/bin/env python
# coding: utf-8
# Distributed under the terms of the MIT License.

""" This submodule is essentially a script to scrape CASTEP help strings
for all possible CASTEP parameters.

"""


def scrape_castep_params(executable):
    """ Scan CASTEP help system for parameter file keywords.

    Parameters:
        executable (str): name of CASTEP executable

    Returns:
        list: list of cell keyword strings
        list: list of cell blocks names
        list: list of param keyword strings
        str: castep version number used

    """
    import string
    import subprocess
    params = set()
    cell = set()
    for letter in string.ascii_lowercase:
        output = str(subprocess.check_output('{} -s {}'.format(executable, letter), shell=True))
        for param in output.split('PARAMETERS')[-1].split('\\n')[1:]:
            try:
                if len(param) > 1:
                    params.add(str(param.split()[0]))
            except Exception:
                pass
        for keyword in output.split('PARAMETERS')[0].split('\\n')[1:]:
            try:
                if len(keyword) > 1:
                    cell.add(str(keyword.split()[0]))
            except Exception:
                pass

    output = str(subprocess.check_output('{} -s block'.format(executable), shell=True))
    blocks = set()
    for keyword in output.split('PARAMETERS')[0].split('\\n')[1:]:
        try:
            if len(keyword) > 1:
                blocks.add(str(keyword.split()[0]))
        except Exception:
            pass

    params = list(sorted(list(params)))
    cell = list(sorted(list(cell)))
    blocks = list(sorted(list(blocks)))
    cell = [keyword for keyword in cell if keyword not in blocks]

    version_string = str(subprocess.check_output('{} --version'
                         .format(executable), shell=True)).split('\\n')[0].split()[-1].strip()
    return cell, blocks, params, version_string


def update_castep_param_list(executable):
    """ Update the castep_params.py file.

    Parameters:
        executable (str): name of CASTEP executable

    """
    cell, blocks, params, version = scrape_castep_params(executable)
    with open('castep_params.py', 'w') as f:
        f.write('""" This file contains a Python list of all CASTEP parameters,\n')
        f.write('automatically generated with file_utils.scrape_castep_params().\n"""')
        f.write('\n\n')
        f.write('CASTEP_VERSION = {}'.format(version))
        f.write('\n\n')
        f.write('CASTEP_CELL_KEYWORDS = [\n')
        for keyword in cell:
            f.write('                 \'{}\',\n'.format(keyword.lower()))
        f.write('                ]\n')

        f.write('CASTEP_CELL_BLOCKS = [\n')
        for keyword in blocks:
            f.write('                 \'{}\',\n'.format(keyword.lower()))
        f.write('                ]\n')

        f.write('CASTEP_PARAMS = [\n')
        for param in params:
            f.write('                 \'{}\',\n'.format(param.lower()))
        f.write('                ]\n')


if __name__ == '__main__':
    from sys import argv
    if len(argv) > 1:
        executable = argv[1]
    else:
        executable = 'castep'
    print('Updating castep_parameters.py with executable {}'.format(executable))
    update_castep_param_list(executable)



# coding: utf-8
# Distributed under the terms of the MIT License.

""" This file implements the diffing of phase diagrams as new structures
are added.

Intended usage:

- Compare current hull with 5 days ago

    matador hulldiff -c KSnP -int --compare 5

- Compare hull a month ago to 2 days ago

    matador hulldiff -c KSnP -int --compare 30 2

"""


from matador.utils.cursor_utils import display_results


class HullDiff:
    """ Takes two QueryConvexHull objects and 'diffs' them, printing a
    +/- line for each new/deleted structure within the specified hull
    cutoff (i.e. compares hull.hull_cursor only).

    """

    def __init__(self, hull_old, hull_new, **args):
        """ Initialise difference from two hulls.

        Parameters:
            hull_old (QueryConvexHull): older hull object.
            hull_new (QueryConvexHull): newer hull object.

        Keyword arguments:
            args: dict, command-line arguments to print results.

        """
        self.id_new = [doc['text_id'] for doc in hull_new.hull_cursor]
        self.id_old = [doc['text_id'] for doc in hull_old.hull_cursor]
        self.additions = []
        self.deletions = []
        self.cursor = []
        # self.cursor = [doc for doc in hull_new.hull_cursor]
        for doc in hull_new.hull_cursor:
            if doc['text_id'] not in self.id_old:
                self.cursor.append(doc)
                self.additions.append(doc['text_id'])
        for doc in hull_old.hull_cursor:
            if doc['text_id'] not in self.id_new:
                self.cursor.append(doc)
                self.deletions.append(doc['text_id'])
        self.cursor = sorted(self.cursor, key=lambda x: (x['enthalpy_per_atom'], x['concentration']))
        self.print_diff(**args)

    def print_diff(self, **args):
        """ Use the display_results function to print the diff. """
        display_results(self.cursor, hull=True, additions=self.additions, deletions=self.deletions)
        if self.additions is not None:
            if args.get('summary'):
                print('{} additions and {} replacements as best at stoichiometry.'
                      .format(len(self.additions), len(self.deletions)))
            else:
                print('{} additions and {} deletions.'.format(len(self.additions), len(self.deletions)))

    def plot_diff(self):
        """ Plot the hull with additions ONLY highlighted in green. """
        raise NotImplementedError


def diff_hulls(client, collections, **args):
    """ Helper function to diff hulls from cmd-line args.

    Parameters:
        client (MongoClient): connection to database
        collections (dict of Collection): dict of collections to query

    Keyword arguments:
        args: dict, command-line arguments to select hulls.

    """
    from matador.query import DBQuery
    from matador.hull import QueryConvexHull
    diff_args = args['compare']
    del args['compare']
    args['no_plot'] = True
    if args.get('verbosity') in [0, None] and not args.get('debug'):
        args['quiet'] = True

    diff_args = sorted(diff_args)
    args['time'] = diff_args[0]
    print('Calculating hull {} days ago...'.format(args['time']))
    query_old = DBQuery(client, collections, **args)
    hull_old = QueryConvexHull(query_old, **args)
    if len(diff_args) == 1 or isinstance(diff_args, str):
        args['time'] = '0'
    else:
        args['time'] = diff_args[1]
    if args['time'] == '0':
        print('... to compare with up-to-date hull.')
    else:
        print('... to compare with hull {} days ago.'.format(args['time']))
    query_new = DBQuery(client, collections, **args)
    hull_new = QueryConvexHull(query_new, **args)
    HullDiff(hull_old, hull_new, **args)
